from django.db import models
from django.conf import settings
from django.utils.translation import gettext_lazy as _


class JwtBerry(models.Model):
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.CASCADE
    )
    jti = models.CharField(unique=True, max_length=255)
    refresh_token = models.CharField(max_length=255)

    created_at = models.DateTimeField(auto_now_add=True)
    expires_at = models.DateTimeField()

    class Meta:
        db_table = 'jwtberries'

        abstract = ('jwtberry.blackberry' not in settings.INSTALLED_APPS)

        verbose_name = _('Jwtberry')
        verbose_name_plural = _('Jwtberries')

    def __str__(self):
        return f'Token for {self.user} - ({self.jti})'


class BlackBerry(models.Model):
    token = models.OneToOneField(
        JwtBerry,
        on_delete=models.CASCADE
    )

    added_at = models.DateTimeField(auto_now_add=True)

    class Meta:
        db_table = 'blackberries'

        abstract = ('jwtberry.blackberry' not in settings.INSTALLED_APPS)

        verbose_name = _('Blackberry')
        verbose_name_plural = _('Blackberries')



from typing import Any

import strawberry
from asgiref.sync import sync_to_async
from django.contrib.auth import get_user_model
from django.contrib.auth.models import update_last_login
from django.utils.module_loading import import_string
from strawberry.types import Info

from jwtberry import settings
from jwtberry.auth import authenticate
from jwtberry.tokens import RefreshToken
from jwtberry.types import JwtAuthError, JwtTokenType


@strawberry.mutation
async def auth_token(info: Info, email: str, password: str) -> Any:
    auth_kwargs = {
        get_user_model().USERNAME_FIELD: email,
        'password': password,
    }

    try:
        auth_kwargs['request'] = info.context.request
    except AttributeError:
        pass

    user = await authenticate(**auth_kwargs)

    auth_rule = import_string(settings.USER_AUTHENTICATION_RULE)
    if not auth_rule(user):
        return JwtAuthError(
            code='auth_error',
            message='Invalid Credentials or inactive user',
        )

    @sync_to_async
    def create_token():
        return RefreshToken.create(user)

    refresh = await create_token()

    data = JwtTokenType(
        access=str(refresh.access_token),
        refresh=str(refresh),
    )

    if settings.UPDATE_LAST_LOGIN:
        update_last_login(None, user)

    return data


@strawberry.mutation
async def refresh_token(info: Info, refresh: str) -> Any:
    ...



from typing import Any, Dict, Optional, Union

from django.utils.translation import gettext_lazy as _


class TokenError(Exception):
    ...


class TokenCoreError(TokenError):
    ...


class JwtAuthFailed(Exception):
    """
    Base class for all JWT Auth exceptions.
    """
    default_detail = _("JWT Auth Failed")
    default_code = "jwt_auth_failed"

    def __init__(
        self,
        detail: Union[str, Dict[str, Any], None] = None,
        code: Optional[str] = None,
    ) -> None:
        self._data = {
            "detail": self.default_detail,
            "code": code or self.default_code,
        }

        if isinstance(detail, dict):
            self._data.update(detail)
        elif isinstance(detail, str):
            self._data["detail"] = detail

        super().__init__(self._data)

    @property
    def data(self) -> Dict[str, Any]:
        return self._data

    def __str__(self) -> str:
        return f'{self._data["detail"]} ({self._data["code"]})'


class InvalidToken(JwtAuthFailed):
    """
    Raised when an invalid token is received.
    """
    default_detail = _("Invalid token")
    default_code = "invalid_token"



from typing import Annotated, Union

import strawberry


@strawberry.type
class JwtTokenType:
    access: str
    refresh: str


@strawberry.type
class JwtAuthError:
    code: str
    message: str


JwtAuthResponse = Annotated[
    Union[JwtTokenType, JwtAuthError],
    strawberry.union("JwtAuthResponse", types=[JwtTokenType, JwtAuthError]),
]



from datetime import timedelta, datetime
from typing import Optional, Any, Dict
from uuid import uuid4

from django.utils.module_loading import import_string
from django.utils.translation import gettext_lazy as _
from django.conf import settings as dj_settings

from jwtberry import settings
from jwtberry.exceptions import TokenError
from jwtberry.helper import aware_utcnow, datetime_from_epoch, datetime_to_epoch

from jwtberry.blackberry.models import JwtBerry, BlackBerry


class Token:

    token_type: str
    lifetime: timedelta
    _token_core: Optional["TokenCore"] = None

    def __init__(
        self,
        token: Optional["Token"] = None,
        verify: bool = True,
    ):
        self.token = token
        self.current_time = aware_utcnow()

        if token is None:
            self.payload = {
                settings.TOKEN_TYPE_CLAIM: self.token_type,
            }
            self.set_exp(from_time=self.current_time, lifetime=self.lifetime)
            self.set_iat(at_time=self.current_time)
            self.set_jti()
        else:
            try:
                self.payload = self.token_core.decode(token, verify=verify)
            except Exception:
                raise ValueError("Token is invalid")

            if verify:
                self.verify()

    def __getitem__(self, key: str):
        return self.payload[key]

    def __setitem__(self, key: str, value: Any):
        self.payload[key] = value

    def __delitem__(self, key: str):
        del self.payload[key]

    def __contains__(self, key: str):
        return key in self.payload

    def __str__(self):
        return self.token_core.encode(self.payload)

    def __repr__(self):
        return f"<{self.__class__.__name__} {self.payload}>"

    def get(self, key: str, default: Optional[Any] = None) -> Optional[Any]:
        return self.payload.get(key, default)

    def verify(self):
        self.verify_exp()

        if settings.TOKEN_TYPE_CLAIM:
            self.verify_type()

        if settings.JTI_CLAIM and settings.JTI_CLAIM not in self.payload:
            raise ValueError("Token has no jti claim")

    def verify_type(self):
        if self.get(settings.TOKEN_TYPE_CLAIM) != self.token_type:
            raise ValueError("Token type is invalid")

    def verify_exp(self, current_time: Optional[datetime] = None):
        current_time = current_time or self.current_time

        if self.get("exp") is None:
            raise ValueError("Token has no expiration")

        claim_time = datetime_from_epoch(self.get("exp"))
        lee_way = self.token_core.get_leeway()
        if claim_time < current_time - lee_way:
            raise ValueError("Token is expired")

    @property
    def token_core(self) -> "TokenCore":
        if self._token_core is None:
            self._token_core = import_string('jwtberry.core.token_core')
        return self._token_core

    def set_exp(
        self,
        from_time: Optional[datetime] = None,
        lifetime: Optional[timedelta] = None,
    ):
        from_time = from_time or self.current_time
        lifetime = lifetime or self.lifetime

        self["exp"] = datetime_to_epoch(from_time + lifetime)

    def set_iat(self, at_time: Optional[datetime] = None):
        at_time = at_time or self.current_time
        self["iat"] = datetime_to_epoch(at_time)

    def set_jti(self):
        self[settings.JTI_CLAIM] = uuid4().hex

    @classmethod
    def create(cls, user) -> "Token":
        token = cls()

        user_id = getattr(user, settings.USER_ID_FIELD)
        if not isinstance(user_id, int):
            user_id = str(user_id)

        token[settings.USER_ID_CLAIM] = user_id
        return token


class BlacklistMixin:
    ...
    payload: Dict[str, Any]

    if "jwtberry.blackberry" in dj_settings.INSTALLED_APPS:

        def verify(self, *args, **kwargs) -> None:
            self.check_blacklist()

            super().verify(*args, **kwargs)  # type: ignore

        def check_blacklist(self) -> None:
            """
            Checks if this token is present in the token blacklist.  Raises
            `TokenError` if so.
            """
            jti = self.payload[settings.JTI_CLAIM]

            if BlackBerry.objects.filter(token__jti=jti).exists():
                raise TokenError(_("Token is blacklisted"))

        def blacklist(self) -> BlackBerry:
            """
            Ensures this token is included in the outstanding token list and
            adds it to the blacklist.
            """
            jti = self.payload[settings.JTI_CLAIM]
            exp = self.payload["exp"]

            # Ensure outstanding token exists with given jti
            token, _ = JwtBerry.objects.get_or_create(
                jti=jti,
                defaults={
                    "token": str(self),
                    "expires_at": datetime_from_epoch(exp),
                },
            )

            return BlackBerry.objects.get_or_create(token=token)

        @classmethod
        def create(cls, user) -> Token:
            """
            Adds this token to the outstanding token list.
            """
            token = super().create(user)  # type: ignore

            jti = token[settings.JTI_CLAIM]
            exp = token["exp"]

            JwtBerry.objects.create(
                user=user,
                jti=jti,
                refresh_token=str(token),
                created_at=token.current_time,
                expires_at=datetime_from_epoch(exp),
            )

            return token


class AccessToken(Token):
    token_type = "access"
    lifetime = settings.ACCESS_TOKEN_LIFETIME


class RefreshToken(BlacklistMixin, Token):
    token_type = "refresh"
    lifetime = settings.REFRESH_TOKEN_LIFETIME
    no_copy_claims = (
        settings.TOKEN_TYPE_CLAIM,
        "exp",
        settings.JTI_CLAIM,
        "jti"
    )
    access_token_class = AccessToken

    @property
    def access_token(self) -> AccessToken:
        access = self.access_token_class()

        access.set_exp(from_time=self.current_time)

        for claim, value in self.payload.items():
            if claim not in self.no_copy_claims:
                access[claim] = value

        return access



import json
from datetime import timedelta
from typing import Optional, Union, Type, Any, Dict

from django.utils.translation import gettext_lazy as _

import jwt
from jwt import (
    PyJWKClient,
    PyJWKClientError,
    InvalidTokenError,
    InvalidAlgorithmError,
    algorithms,
)

from jwtberry import settings
from jwtberry.exceptions import TokenCoreError
from jwtberry.helper import format_lazy
from jwtberry.tokens import Token


class TokenCore:

    def __init__(
            self,
            algorithm: settings.Algorithm,
            signing_key: Optional[str] = None,
            verifying_key: str = '',
            audience: Optional[str] = None,
            issuer: Optional[str] = None,
            jwk_url: Optional[str] = None,
            leeway: Union[int, float, timedelta] = None,
            json_encoder: Optional[Type[json.JSONEncoder]] = None,
    ):
        self._validate_algorithm(algorithm)

        self.algorithm = algorithm
        self.signing_key = signing_key
        self.verifying_key = verifying_key
        self.audience = audience
        self.issuer = issuer

        self.jwk_client = PyJWKClient(jwk_url) if jwk_url else None
        self.leeway = leeway
        self.json_encoder = json_encoder

    @staticmethod
    def _validate_algorithm(algorithm: settings.Algorithm):
        if algorithm not in settings.Algorithm:
            raise TokenCoreError(format_lazy(_("Algorithm '{}' is not supported"), algorithm))

        if algorithm in algorithms.requires_cryptography and not algorithms.has_crypto:
            raise TokenCoreError(
                format_lazy(
                    _("Algorithm '{}' requires cryptography package"), algorithm
                )
            )

    def encode(self, payload: Dict[str, Any]) -> str:
        """Encode a payload into a JWT string"""

        jwt_payload = payload.copy()
        if self.audience is not None:
            jwt_payload["aud"] = self.audience

        if self.issuer is not None:
            jwt_payload["iss"] = self.issuer

        token = jwt.encode(
            jwt_payload,
            self.signing_key,
            algorithm=self.algorithm.value,
            json_encoder=self.json_encoder,
        )
        if isinstance(token, bytes):
            return token.decode("utf-8")
        return token

    def decode(self, token: Token, verify: bool = True) -> Dict[str, Any]:
        """Decode a JWT string into a payload dict"""

        try:
            return jwt.decode(
                token,
                self.get_verifying_key(token),
                algorithms=[self.algorithm.value],
                audience=self.audience,
                issuer=self.issuer,
                leeway=self.get_leeway(),
                options={
                    "verify_signature": verify,
                    "verify_aud": self.audience is not None,
                },
            )
        except InvalidAlgorithmError as exc:
            raise TokenCoreError(
                format_lazy(_("Algorithm '{}' is not supported"), self.algorithm)
            ) from exc
        except InvalidTokenError as exc:
            raise TokenCoreError(_("Token is invalid or expired")) from exc

    def get_verifying_key(self, token):
        if self.algorithm.value.startswith("HS"):
            return self.signing_key

        if self.jwk_client:
            try:
                return self.jwk_client.get_signing_key_from_jwt(token).key
            except PyJWKClientError as exc:
                raise TokenCoreError(_("Unable to load JWK keys")) from exc
        return self.verifying_key

    def get_leeway(self):
        if self.leeway is None:
            return timedelta(seconds=0)
        if isinstance(self.leeway, (int, float)):
            return timedelta(seconds=self.leeway)
        if isinstance(self.leeway, timedelta):
            return self.leeway

        raise TokenCoreError(
            format_lazy(
                _("Unsupported leeway type '{}'. Expected int, float or timedelta"),
                type(self.leeway),
            )
        )


token_core = TokenCore(
    algorithm=settings.ALGORITHM,
    signing_key=settings.SIGNING_KEY,
    verifying_key=settings.VERIFYING_KEY,
    audience=settings.AUDIENCE,
    issuer=settings.ISSUER,
    jwk_url=settings.JWK_URL,
    leeway=settings.LEEWAY,
    json_encoder=settings.JSON_ENCODER,
)



import asyncio
import inspect
import re

from asgiref.sync import sync_to_async
from django.contrib.auth import get_backends, user_login_failed
from django.core.exceptions import PermissionDenied
from django.core.handlers.asgi import ASGIRequest
from django.views.decorators.debug import sensitive_variables

SENSITIVE_CREDENTIALS = re.compile("api|token|key|secret|password|signature", re.I)
CLEANSED_SUBSTITUTE = "********************"


@sensitive_variables("credentials")
def _clean_credentials(credentials):
    """
    Clean a dictionary of credentials of potentially sensitive info before
    sending to less secure functions.

    Not comprehensive - intended for user_login_failed signal
    """
    for key in credentials:
        if SENSITIVE_CREDENTIALS.search(key):
            credentials[key] = CLEANSED_SUBSTITUTE
    return credentials


@sensitive_variables("credentials")
async def authenticate(request=None, **credentials):
    """
    If the given credentials are valid, return a User object.
    """
    for backend in get_backends():
        backend_signature = inspect.signature(backend.authenticate)
        try:
            backend_signature.bind(request, **credentials)
        except TypeError:
            # This backend doesn't accept these credentials as arguments. Try the next one.
            continue
        try:
            if hasattr(backend, "authenticate_async"):
                user = await backend.authenticate_async(request, **credentials)
            elif asyncio.iscoroutinefunction(backend.authenticate):
                user = await backend.authenticate(request, **credentials)
            else:
                # if isinstance(request, ASGIRequest):
                #     user = await sync_to_async(backend.authenticate)(request, **credentials)
                # else:
                user = await sync_to_async(backend.authenticate)(request, **credentials)
        except PermissionDenied:
            # This backend says to stop in our tracks - this user should not be allowed in at all.
            break
        if user is None:
            continue
        # Annotate the user object with the path of the backend.
        user.backend = f"{backend.__module__}.{backend.__class__.__qualname__}"
        return user

    # The credentials supplied are invalid to all backends, fire signal
    user_login_failed.send(sender=__name__, credentials=_clean_credentials(credentials), request=request)

    return None



from datetime import timedelta
import enum
from typing import Tuple, Optional

from django.conf import settings


class Algorithm(enum.Enum):
    HS256 = "HS256"
    HS384 = "HS384"
    HS512 = "HS512"
    RS256 = "RS256"
    RS384 = "RS384"
    RS512 = "RS512"
    ES256 = "ES256"
    ES384 = "ES384"
    ES512 = "ES512"


ACCESS_TOKEN_LIFETIME: timedelta = timedelta(minutes=5)
REFRESH_TOKEN_LIFETIME: timedelta = timedelta(days=1)
UPDATE_LAST_LOGIN: bool = False
ALGORITHM: Algorithm = Algorithm.HS256
SIGNING_KEY: str = settings.SECRET_KEY
VERIFYING_KEY: str = ''
AUDIENCE: Optional[str] = None
ISSUER: Optional[str] = None
JSON_ENCODER: Optional[str] = None
JWK_URL: Optional[str] = None
LEEWAY: timedelta = timedelta(seconds=0)
AUTH_HEADER_TYPES: Tuple[str] = ('Bearer',)
AUTH_HEADER_NAME: str = "Authorization"
USER_ID_FIELD: str = 'id'
USER_ID_CLAIM: str = 'user_id'
USER_AUTHENTICATION_RULE: str = 'jwtberry.backends.default_user_authentication_rule'
AUTH_TOKEN_CLASSES: Tuple[str] = ('jwtberry.tokens.AccessToken',)
TOKEN_TYPE_CLAIM: str = 'token_type'
JTI_CLAIM: str = 'jti'
# TOKEN_USER_CLASS: str = "rest_framework_simplejwt.models.TokenUser"



from calendar import timegm
from datetime import datetime, timezone
from typing import Callable, Any, Set

from django.conf import settings
from django.http import HttpRequest
from django.utils.functional import lazy
from django.utils.timezone import is_naive, make_aware
from graphql import GraphQLResolveInfo
from strawberry.django.context import StrawberryDjangoContext
from strawberry.types import Info, ExecutionContext
from jwtberry.settings import AUTH_HEADER_TYPES, AUTH_HEADER_NAME


def make_utc(dt: datetime) -> datetime:
    if settings.USE_TZ and is_naive(dt):
        return make_aware(dt, timezone=timezone.utc)

    return dt


def aware_utcnow() -> datetime:
    return make_utc(datetime.utcnow())


def datetime_to_epoch(dt: datetime) -> int:
    return timegm(dt.utctimetuple())


def datetime_from_epoch(ts: float) -> datetime:
    return make_utc(datetime.utcfromtimestamp(ts))


def format_lazy(s: str, *args, **kwargs) -> str:
    return s.format(*args, **kwargs)


format_lazy: Callable = lazy(format_lazy, str)


AUTH_HEADER_TYPE_BYTES: Set[bytes] = {
    h.encode('utf-8') for h in AUTH_HEADER_TYPES
}


def get_http_authorization(request: HttpRequest) -> str | None:
    header = request.headers.get(AUTH_HEADER_NAME)
    if isinstance(header, str):
        header = header.encode("utf-8")

    if header is None:
        return None

    auth = header.split()

    if len(auth) != 2 or auth[0] not in AUTH_HEADER_TYPE_BYTES:
        return None
    return auth[1].decode("utf-8")



from typing import Union, Optional, List, Any

from django.contrib.auth import models as auth_models
from django.db.models.manager import EmptyManager
from django.utils.functional import cached_property

from jwtberry import settings


class JwtUser:

    is_active: bool = True

    _groups = EmptyManager(auth_models.Group)
    _user_permissions = EmptyManager(auth_models.Permission)

    def __init__(self, token: "Token"):
        self.token = token

    def __str__(self):
        return str(self.token)

    def __repr__(self):
        return repr(self.token)

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, JwtUser):
            return NotImplemented

        return self.id == other.id

    def __ne__(self, other: object) -> bool:
        return not self.__eq__(other)

    def __hash__(self) -> int:
        return hash(self.id)

    def __getattr__(self, attr: str) -> Optional[Any]:
        return self.token.get(attr, None)

    @cached_property
    def id(self) -> Union[int, str]:
        return self.token[settings.USER_ID_CLAIM]

    @cached_property
    def pk(self) -> Union[int, str]:
        return self.id

    @cached_property
    def username(self) -> str:
        return self.token.get("username", "")

    @cached_property
    def is_staff(self) -> bool:
        return self.token.get("is_staff", False)

    @cached_property
    def is_superuser(self) -> bool:
        return self.token.get("is_superuser", False)

    @property
    def is_anonymous(self) -> bool:
        return False

    @property
    def is_authenticated(self) -> bool:
        return True

    @property
    def get_username(self) -> str:
        return self.username

    @property
    def groups(self) -> auth_models.Group:
        return self._groups

    @property
    def user_permissions(self) -> auth_models.Permission:
        return self._user_permissions

    def get_group_permissions(self, obj: Optional[object] = None) -> set:
        return set()

    def get_all_permissions(self, obj: Optional[object] = None) -> set:
        return set()

    def has_perm(self, perm: str, obj: Optional[object] = None) -> bool:
        return False

    def has_perms(self, perm_list: List[str], obj: Optional[object] = None) -> bool:
        return False

    def has_module_perms(self, module: str) -> bool:
        return False

    def save(self) -> None:
        raise NotImplementedError("Token users have no DB representation")

    def delete(self) -> None:
        raise NotImplementedError("Token users have no DB representation")

    def set_password(self, raw_password: str) -> None:
        raise NotImplementedError("Token users have no DB representation")

    def check_password(self, raw_password: str) -> None:
        raise NotImplementedError("Token users have no DB representation")



from functools import cached_property

from asgiref.sync import sync_to_async
from django.contrib.auth.models import User
from strawberry.django.context import StrawberryDjangoContext

from jwtberry.helper import get_http_authorization
from django.contrib.auth import authenticate


class JwtContext(StrawberryDjangoContext):

    @cached_property
    def user(self) -> User | None:
        if not self.request:
            return None

        return authenticate(request=self.request)




from typing import Any

from django.http import HttpResponse, HttpRequest
from strawberry.django.views import GraphQLView, AsyncGraphQLView

from jwtberry.context import JwtContext


class JwtGraphQLView(GraphQLView):
    """
        This is not implemented yet, please use JwtAsyncGraphQLView
    """
    def get_context(self, request: HttpRequest, response: HttpResponse) -> Any:
        ...
        # return JwtContext(request=request, response=response)


class JwtAsyncGraphQLView(AsyncGraphQLView):

    async def get_context(self, request: HttpRequest, response: HttpResponse) -> Any:
        return JwtContext(request=request, response=response)





from django.contrib import admin

from .models import JwtBerry, BlackBerry


@admin.register(JwtBerry)
class JwtBerryAdmin(admin.ModelAdmin):
    list_display = (
        'jti',
        'user',
        'created_at',
        'expires_at',
    )

    search_fields = (
        'user__email',
        'user__id',
        'jti',
    )
    ordering = ('-created_at',)

    actions = None

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def has_change_permission(self, request, obj=None):
        return request.method in ['GET', 'HEAD', 'OPTIONS'] or super().has_change_permission(request, obj)

    def get_readonly_fields(self, request, obj=None):
        return self.fields or [f.name for f in self.model._meta.fields]


@admin.register(BlackBerry)
class BlackBerryAdmin(admin.ModelAdmin):
    list_display = (
        'jti',
        'user',
        'created_at',
        'expires_at',
        'added_at',
    )
    search_fields = (
        'token__user__email',
        'token__user__id',
        'token__jti',
    )

    ordering = ('-added_at',)

    def get_queryset(self, *args, **kwargs):
        qs = super().get_queryset(*args, **kwargs)
        return qs.select_related('token__user')

    @admin.display(description='User')
    def user(self, obj):
        return obj.token.user

    @admin.display(description='JTI')
    def jti(self, obj):
        return obj.token.jti

    @admin.display(description='Created At')
    def created_at(self, obj):
        return obj.token.created_at

    @admin.display(description='Expires At')
    def expires_at(self, obj):
        return obj.token.expires_at



from typing import Optional, Set, TypeVar

from asgiref.sync import sync_to_async
from django.contrib.auth.base_user import AbstractBaseUser
from django.utils.module_loading import import_string
from django.utils.translation import gettext_lazy as _
from django.contrib.auth import get_user_model

from jwtberry import settings
from jwtberry.exceptions import InvalidToken, JwtAuthFailed
from jwtberry.models import JwtUser
from jwtberry.tokens import Token


User = TypeVar("User", AbstractBaseUser, JwtUser)


class JwtAuthBackend:
    user_model = get_user_model()

    AUTH_HEADER_TYPE_BYTES: Set[bytes] = {
        h.encode('utf-8') for h in settings.AUTH_HEADER_TYPES
    }

    def authenticate(self, request=None, **kwargs):
        if request is None or hasattr(request, "__jwtberry__"):
            return None

        # if hasattr(request, "user") and request.user.is_authenticated:
        #     return request.user

        header = self.get_header(request)
        if header is None:
            return None

        raw_token = self.get_raw_token(header)
        if raw_token is None:
            return None

        validated_token = self.get_validated_token(raw_token)
        return self.get_user(validated_token)

    async def authenticate_async(self, request=None, **kwargs):
        return await sync_to_async(self.authenticate)(request, **kwargs)

    @staticmethod
    def get_header(request) -> Optional[bytes]:
        header = request.headers.get(settings.AUTH_HEADER_NAME)
        if isinstance(header, str):
            header = header.encode("utf-8")
        return header

    def get_raw_token(self, header: bytes) -> Optional[bytes]:
        parts = header.split()
        if len(parts) == 0:
            return None

        if parts[0] not in self.AUTH_HEADER_TYPE_BYTES:
            return None

        if len(parts) != 2:
            raise JwtAuthFailed({
                "detail": _("Invalid Authorization header. Credentials string should not contain spaces."),
                "code": "invalid_authorization_header",
            })

        return parts[1]

    def get_validated_token(self, raw_token: bytes) -> Token:
        messages = []
        for TokenClass in (import_string(tm) for tm in settings.AUTH_TOKEN_CLASSES):
            try:
                return TokenClass(raw_token, verify=False)
            except ValueError as e:
                messages.append({
                    "token_class": TokenClass.__name__,
                    "token_type": TokenClass.token_type,
                    "message": str(e),
                })

        raise InvalidToken({
            "detail": _("Token is invalid or expired"),
            "messages": messages,
        })

    def get_user(self, validated_token: Token):
        if not isinstance(validated_token, Token):
            return None

        try:
            user_id = validated_token[settings.USER_ID_CLAIM]
        except KeyError:
            raise InvalidToken({
                "detail": _("Token contained no recognizable user identification"),
                "code": "invalid_user_id_claim",
            })

        try:
            user = self.user_model.objects.get(**{settings.USER_ID_FIELD: user_id})
        except self.user_model.DoesNotExist:
            raise InvalidToken({
                "detail": _("User not found"),
                "code": "user_not_found",
            })

        if not user.is_active:
            raise InvalidToken({
                "detail": _("User is inactive"),
                "code": "user_inactive",
            })
        return user


def default_user_authentication_rule(user: User) -> bool:
    return user is not None and user.is_active



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class JwtTokenConfig(AppConfig):
    name = "jwtberry.blackberry"
    verbose_name = _("JWT Token")
    default_auto_field = "django.db.models.BigAutoField"



import asyncio
from typing import Any

from asgiref.sync import sync_to_async
from strawberry import BasePermission
from strawberry.types import Info


class IsAuthenticated(BasePermission):
    message = "User is not authenticated"

    # This method can also be sync!
    async def has_permission(self, source: Any, info: Info, **kwargs) -> bool:
        @sync_to_async
        def check():
            return info.context.user and info.context.user.is_authenticated

        return await check()
